@@ -1,32 +1,30 @@
 /*
-    parse.c: parse CCID structure
-    Copyright (C) 2003-2010   Ludovic Rousseau
+	parse.c: parse CCID structure
+	Copyright (C) 2003-2010   Ludovic Rousseau
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
 	You should have received a copy of the GNU General Public License along
 	with this program; if not, write to the Free Software Foundation, Inc., 51
 	Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/*
- * $Id: parse.c 5599 2011-02-16 12:52:16Z rousseau $
- */
-
 #include <stdio.h>
 #include <string.h>
 # ifdef S_SPLINT_S
 # include <sys/types.h>
 # endif
 #include <errno.h>
+#include <unistd.h>
+#include <stdbool.h>
 
 #include "defs.h"
 #include "ccid.h"
@@ -35,11 +33,6 @@
  * returned by bNumClockSupported and bNumDataRatesSupported */
 #undef DISPLAY_EXTRA_VALUES
 
-#ifndef TRUE
-#define TRUE 1
-#define FALSE 0
-#endif
-
 #define BLUE "\33[34m"
 #define RED "\33[31m"
 #define BRIGHT_RED "\33[01;31m"
@@ -51,7 +44,7 @@
 int LogLevel = 1+2+4+8; /* full debug */
 int DriverOptions = 0;
 
-static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
+static bool ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	struct libusb_device_descriptor desc,
 	struct libusb_config_descriptor *config_desc,
 	int num,
@@ -68,24 +61,25 @@ int main(int argc, char *argv[])
 	libusb_device **devs, *dev;
 	int nb = 0, r, i;
 	unsigned char buffer[256];
-	char class_ff = FALSE;
+	bool class_ff = false;
 	ssize_t cnt;
 
 	if ((argc > 1) && (0 == strcmp(argv[1], "-p")))
-		class_ff = TRUE;
+		class_ff = true;
 
 	r = libusb_init(NULL);
 	if (r < 0)
 	{
-		(void)printf("libusb_init() failed\n");
-        return r;
+		(void)printf("libusb_init() failed: %s\n", libusb_error_name(r));
+		return r;
 	}
 
 	cnt = libusb_get_device_list(NULL, &devs);
-    if (cnt < 0)
+	if (cnt < 0)
 	{
-		(void)printf("libusb_get_device_list() failed\n");
-        return (int)cnt;
+		(void)printf("libusb_get_device_list() failed: %s\n",
+			libusb_error_name(r));
+		return (int)cnt;
 	}
 
 	/* for every device */
@@ -96,13 +90,16 @@ int main(int argc, char *argv[])
 		struct libusb_config_descriptor *config_desc;
 		struct libusb_device_handle *handle;
 		const struct libusb_interface *usb_interface = NULL;
+#ifndef __APPLE__
 		int interface;
+#endif
 		int num = 0;
 
 		r = libusb_open(dev, &handle);
 		if (r < 0)
 		{
-			(void)fprintf(stderr, "Can't libusb_open(): %s\n", strerror(errno));
+			(void)fprintf(stderr, "Can't libusb_open(): %s\n",
+				libusb_error_name(r));
 			if (getuid())
 			{
 				(void)fprintf(stderr,
@@ -113,24 +110,26 @@ int main(int argc, char *argv[])
 		}
 
 		r = libusb_get_device_descriptor(dev, &desc);
-        if (r < 0)
+		if (r < 0)
 		{
-            (void)fprintf(stderr,
-				BRIGHT_RED "failed to get device descriptor" NORMAL);
-            return 1;
-        }
+			(void)fprintf(stderr,
+				BRIGHT_RED "failed to get device descriptor: %s" NORMAL,
+				libusb_error_name(r));
+			return 1;
+		}
 
 		(void)fprintf(stderr,
 			"Parsing USB bus/device: %04X:%04X (bus %d, device %d)\n",
 			desc.idVendor, desc.idProduct,
-            libusb_get_bus_number(dev), libusb_get_device_address(dev));
+			libusb_get_bus_number(dev), libusb_get_device_address(dev));
 
 		(void)fprintf(stderr, " idVendor:  0x%04X", desc.idVendor);
 		r = libusb_get_string_descriptor_ascii(handle, desc.iManufacturer,
 			buffer, sizeof(buffer));
 		if (r < 0)
 		{
-			(void)fprintf(stderr, "  Can't get iManufacturer string\n");
+			(void)fprintf(stderr, "  Can't get iManufacturer string: %s\n",
+				libusb_error_name(r));
 			if (getuid())
 			{
 				(void)fprintf(stderr,
@@ -146,7 +145,8 @@ int main(int argc, char *argv[])
 		r = libusb_get_string_descriptor_ascii(handle, desc.iProduct,
 			buffer, sizeof(buffer));
 		if (r < 0)
-			(void)fprintf(stderr, "  Can't get iProduct string\n");
+			(void)fprintf(stderr, "  Can't get iProduct string: %s\n",
+				libusb_error_name(r));
 		else
 			(void)fprintf(stderr, "  iProduct: " BLUE "%s\n" NORMAL, buffer);
 
@@ -155,7 +155,9 @@ again:
 		r = libusb_get_active_config_descriptor(dev, &config_desc);
 		if (r < 0)
 		{
-			(void)fprintf(stderr, "  Can't get config descriptor: %d\n", r);
+			(void)fprintf(stderr, "  Can't get config descriptor: %s\n",
+				libusb_error_name(r));
+			(void)libusb_close(handle);
 			continue;
 		}
 
@@ -170,7 +172,8 @@ again:
 		}
 		if (!class_ff && (0xFF == usb_interface->altsetting->bInterfaceClass))
 		{
-			(void)fprintf(stderr, MAGENTA "  Found a possibly CCID/ICCD device (bInterfaceClass = 0xFF). Use -p\n" NORMAL);
+			(void)libusb_close(handle);
+			(void)fprintf(stderr, MAGENTA "  Found a possibly CCID/ICCD device (bInterfaceClass = 0xFF). Use %s -p\n" NORMAL, argv[0]);
 			continue;
 		}
 		(void)fprintf(stderr,
@@ -187,22 +190,28 @@ again:
 		}
 #endif
 
-		interface = usb_interface->altsetting->bInterfaceNumber;
 #ifndef __APPLE__
+		interface = usb_interface->altsetting->bInterfaceNumber;
 		r = libusb_claim_interface(handle, interface);
 		if (r < 0)
 		{
 			(void)fprintf(stderr,
 				"Can't claim interface (bus %d, device %d): %s\n",
 				libusb_get_bus_number(dev), libusb_get_device_address(dev),
-				strerror(errno));
+				libusb_error_name(r));
 			(void)libusb_close(handle);
 
 			if (EBUSY == errno)
 			{
 				(void)fprintf(stderr,
 					BRIGHT_RED " Please, stop pcscd and retry\n\n" NORMAL);
-				return TRUE;
+
+				if (class_ff)
+					/* maybe the device with Class = 0xFF is NOT a CCID
+					 * reader */
+					continue;
+				else
+					return true;
 			}
 			continue;
 		}
@@ -210,6 +219,7 @@ again:
 
 		(void)ccid_parse_interface_descriptor(handle, desc, config_desc, num,
 			usb_interface);
+		nb++;
 
 #ifndef __APPLE__
 		(void)libusb_release_interface(handle, interface);
@@ -217,9 +227,6 @@ again:
 		/* check for another CCID interface on the same device */
 		num++;
 		goto again;
-
-		(void)libusb_close(handle);
-		nb++;
 	}
 
 	if ((0 == nb) && (0 != geteuid()))
@@ -237,8 +244,8 @@ again:
  *					Parse a CCID USB Descriptor
  *
  ****************************************************************************/
-static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
-	struct libusb_device_descriptor desc, 
+static bool ccid_parse_interface_descriptor(libusb_device_handle *handle,
+	struct libusb_device_descriptor desc,
 	struct libusb_config_descriptor *config_desc,
 	int num,
 	const struct libusb_interface *usb_interface)
@@ -256,12 +263,13 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 		buffer, sizeof(buffer));
 	if (r < 0)
 	{
-		(void)printf("  Can't get iManufacturer string\n");
+		(void)printf("  Can't get iManufacturer string: %s\n",
+			libusb_error_name(r));
 		if (getuid())
 		{
 			(void)fprintf(stderr,
 				BRIGHT_RED "Please, restart the command as root\n\n" NORMAL);
-			return TRUE;
+			return true;
 		}
 	}
 	else
@@ -271,7 +279,8 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	r =  libusb_get_string_descriptor_ascii(handle, desc.iProduct,
 		buffer, sizeof(buffer));
 	if (r < 0)
-		(void)printf("  Can't get iProduct string\n");
+		(void)printf("  Can't get iProduct string: %s\n",
+			libusb_error_name(r));
 	else
 		(void)printf("  iProduct: %s\n", buffer);
 
@@ -314,7 +323,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	{
 		(void)printf("\n  NOT A CCID DEVICE\n");
 		if (usb_interface_descriptor->bInterfaceClass != 0xFF)
-			return TRUE;
+			return true;
 		else
 			(void)printf("  Class is 0xFF (proprietary)\n");
 	}
@@ -344,7 +353,8 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	r = libusb_get_string_descriptor_ascii(handle, usb_interface_descriptor->iInterface,
 		buffer, sizeof(buffer));
 	if (r < 0)
-		(void)printf(" Can't get iInterface string\n");
+		(void)printf(" Can't get iInterface string: %s\n",
+			libusb_error_name(r));
 	else
 		(void)printf(" iInterface: %s\n", buffer);
 
@@ -352,7 +362,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	if (NULL == device_descriptor)
 	{
 		(void)printf("\n  NOT A CCID DEVICE\n");
-		return TRUE;
+		return true;
 	}
 
 	/*
@@ -364,7 +374,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	if (device_descriptor[0] != 0x36)
 	{
 		(void)printf("   UNSUPPORTED bLength\n");
-		return TRUE;
+		return true;
 	}
 
 	(void)printf("  bDescriptorType: 0x%02X\n", device_descriptor[1]);
@@ -375,7 +385,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 		else
 		{
 			(void)printf("   UNSUPPORTED bDescriptorType\n");
-			return TRUE;
+			return true;
 		}
 	}
 
@@ -398,49 +408,54 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 
 	(void)printf("  dwDefaultClock: %.3f MHz\n", dw2i(device_descriptor, 10)/1000.0);
 	(void)printf("  dwMaximumClock: %.3f MHz\n", dw2i(device_descriptor, 14)/1000.0);
-	(void)printf("  bNumClockSupported: %d%s\n", device_descriptor[18],
-		device_descriptor[18] ? "" : " (will use whatever is returned)");
+	int bNumClockSupported = device_descriptor[18];
+	(void)printf("  bNumClockSupported: %d%s\n", bNumClockSupported,
+		bNumClockSupported ? "" : " (will use whatever is returned)");
 	{
 		int n;
 
-		/* See CCID 3.7.2 page 25 */
+		if (0 == bNumClockSupported)
+			/* read up to the buffer size */
+			bNumClockSupported = sizeof(buffer) / sizeof(int);
+
+		/* See CCID 5.3.2 page 24 */
 		n = libusb_control_transfer(handle,
 			0xA1, /* request type */
 			0x02, /* GET CLOCK FREQUENCIES */
 			0x00, /* value */
 			usb_interface_descriptor->bInterfaceNumber, /* interface */
 			buffer,
-			sizeof(buffer),
+			bNumClockSupported * sizeof(int),
 			2 * 1000);
 
 		/* we got an error? */
 		if (n <= 0)
 		{
-			(void)(void)printf("   IFD does not support GET CLOCK FREQUENCIES request: %s\n", strerror(errno));
+			(void)(void)printf("   IFD does not support GET CLOCK FREQUENCIES request: %s\n", libusb_error_name(n));
 			if (EBUSY == errno)
 			{
 				(void)fprintf(stderr,
 					BRIGHT_RED "   Please, stop pcscd and retry\n\n" NORMAL);
-				return TRUE;
+				return true;
 			}
 		}
 		else
-			if (n % 4) 	/* not a multiple of 4 */
+			if (n % 4)	/* not a multiple of 4 */
 				(void)printf("   wrong size for GET CLOCK FREQUENCIES: %d\n", n);
 			else
 			{
 				int i;
 
 				/* we do not get the expected number of data rates */
-				if ((n != device_descriptor[18]*4) && device_descriptor[18])
+				if ((n != bNumClockSupported*4) && bNumClockSupported)
 				{
 					(void)printf("   Got %d clock frequencies but was expecting %d\n",
-						n/4, device_descriptor[18]);
+						n/4, bNumClockSupported);
 
 					/* we got more data than expected */
 #ifndef DISPLAY_EXTRA_VALUES
-					if (n > device_descriptor[18]*4)
-						n = device_descriptor[18]*4;
+					if (n > bNumClockSupported*4)
+						n = bNumClockSupported*4;
 #endif
 				}
 
@@ -450,42 +465,50 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	}
 	(void)printf("  dwDataRate: %d bps\n", dw2i(device_descriptor, 19));
 	(void)printf("  dwMaxDataRate: %d bps\n", dw2i(device_descriptor, 23));
-	(void)printf("  bNumDataRatesSupported: %d%s\n", device_descriptor[27],
-		device_descriptor[27] ? "" : " (will use whatever is returned)");
+	int bNumDataRatesSupported = device_descriptor[27];
+	(void)printf("  bNumDataRatesSupported: %d%s\n", bNumDataRatesSupported,
+		bNumDataRatesSupported ? "" : " (will use whatever is returned)");
 	{
 		int n;
+		int n_max;
 
-		/* See CCID 3.7.3 page 25 */
+		if (0 == bNumDataRatesSupported)
+			/* read up to the buffer size */
+			n_max = sizeof(buffer) / sizeof(int);
+		else
+			n_max = bNumDataRatesSupported;
+
+		/* See CCID 5.3.3 page 24 */
 		n = libusb_control_transfer(handle,
 			0xA1, /* request type */
 			0x03, /* GET DATA RATES */
 			0x00, /* value */
 			usb_interface_descriptor->bInterfaceNumber, /* interface */
 			buffer,
-			sizeof(buffer),
+			n_max * sizeof(int),
 			2 * 1000);
 
 		/* we got an error? */
 		if (n <= 0)
 			(void)printf("   IFD does not support GET_DATA_RATES request: %s\n",
-				strerror(errno));
+				libusb_error_name(n));
 		else
-			if (n % 4) 	/* not a multiple of 4 */
+			if (n % 4)	/* not a multiple of 4 */
 				(void)printf("   wrong size for GET_DATA_RATES: %d\n", n);
 			else
 			{
 				int i;
 
 				/* we do not get the expected number of data rates */
-				if ((n != device_descriptor[27]*4) && device_descriptor[27])
+				if ((n != bNumDataRatesSupported*4) && bNumDataRatesSupported)
 				{
 					(void)printf("   Got %d data rates but was expecting %d\n", n/4,
-						device_descriptor[27]);
+						bNumDataRatesSupported);
 
 					/* we got more data than expected */
 #ifndef DISPLAY_EXTRA_VALUES
-					if (n > device_descriptor[27]*4)
-						n = device_descriptor[27]*4;
+					if (n > bNumDataRatesSupported*4)
+						n = bNumDataRatesSupported*4;
 #endif
 				}
 
@@ -538,7 +561,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	if (device_descriptor[41] & 0x04)
 		(void)printf("   ..04.. Automatic IFSD exchange as first exchange (T=1)\n");
 	if (device_descriptor[41] & 0x08)
-		(void)printf("   ..08.. Unknown (ICCD?)\n");
+		(void)printf("   ..08.. ICCD token\n");
 	switch (device_descriptor[42] & 0x07)
 	{
 		case 0x00:
@@ -564,7 +587,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	(void)printf("  bClassGetResponse: 0x%02X\n", device_descriptor[48]);
 	if (0xFF == device_descriptor[48])
 		(void)printf("   echoes the APDU class\n");
-	(void)printf("  bClassEnveloppe: 0x%02X\n", device_descriptor[49]);
+	(void)printf("  bClassEnvelope: 0x%02X\n", device_descriptor[49]);
 	if (0xFF == device_descriptor[49])
 		(void)printf("   echoes the APDU class\n");
 	(void)printf("  wLcdLayout: 0x%04X\n", (device_descriptor[51] << 8)+device_descriptor[50]);
@@ -579,6 +602,6 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 		(void)printf("   PIN Modification supported\n");
 	(void)printf("  bMaxCCIDBusySlots: %d\n", device_descriptor[53]);
 
-	return FALSE;
+	return false;
 } /* ccid_parse_interface_descriptor */
 
